<?php

namespace DNB;

use Reflection;
use ReflectionClass;
use ReflectionFunction;

abstract class AbstractPostSerializable implements PostSerializableInterface {
    public static function fromPostParams(array $post): PostSerializableInterface {
        $vars = [];

        foreach (static::getPostParams() as $key => $required) {
            $vars[$key] = isset($post[$key]) && ((bool)$post[$key]) && strtolower((string)$post[$key]) != "off" ? htmlspecialchars($post[$key], ENT_QUOTES | ENT_SUBSTITUTE) : null;

            if ($vars[$key] == null && $required)
                throw new \Symfony\Component\HttpClient\Exception\InvalidArgumentException(
                    sprintf("Missing required argmuent: %s", $key)
                );
        }

        return new static(...array_values($vars));
    }

    protected static function removePrefix(string $param): string {
        return ltrim(strstr($param, '-'), '-');
    }

    final public static function getPostParams(bool $short = false): array {
        
        $vars = [];

        foreach ((new ReflectionClass(static::class))->getConstructor()->getParameters() as $param) {
            if ($short)
                $vars[$param->getName()] = !($param->isOptional() || $param->allowsNull());
            else
                $vars[sprintf('%s-%s', strtolower(basename(str_replace('\\', DIRECTORY_SEPARATOR, static::class))), $param->getName())] = !($param->isOptional() || $param->allowsNull());
        }

        return $vars;
    }

}
